/*
	defs.h
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

struct vct //A vector
{
	long xx;
	long yy; //X and Y components
};

struct pol //A polar vector
{
        short ang; //Angle
	long rad; //Radius
};

class frag //A frag object, like phaser fire, torpedo, debris
{
	public:
	
	short self; //Self index in the database
	short typ; //Type: -1 Non-existent  1 Phaser fire  2 Torpedo  3 Debris
	short spr; //Associated sprite
	short trg; //Target ship index
	short own; //Owner ship index
	long x,y; //Position
	vct vel; //Velocity
	short rot; //Rotation frame to use
	short pow; //Power/damage
	long trck; //Tracking power
	short rng; //Range left
	
	void draw(); //Draw the frag
	void physics(); //Move the frag, do collisions
	void home(); //Sub-function of physics, handles homing
	void colldetect(); //Sub-function of physics, handles collision detection
	void save(FILE* strm); //Save frag to a filestream
	void load(FILE* strm); //Load frag from a filestream
};

class ship //Definition of the spaceship type in game
{
	public:
	
	short self; //Self index in the ship database
	short typ; //Ship type index
	short ply; //Is a player?
	char cls[65]; //Ship class 
	short spr; //Sprite index
	short sspr; //Shield sprite
	short ssnd; //Shield sound
	short fspr; //Frag sprite
	short fsnd; //Frag sound
	short all; //Alliance (0 means dead)
	short stat; //Status: 0 - Disabled  1 - Holding  2 - Active
	long x,y; //Co-ordinates 
	pol vel; //Velocity
	short trn; //Turning speed
	long mip; //Maximum impulse velocity (<100)
	long aip; //Impulse acceleration
	long mwp; //Maximum warp velocity
	long awp; //Warp acceleration
	long srng; //Sensor range
	short hul; //Hull integrity (related to mass)
	short mss; //Mass
	short pow; //Current power level
	short rpow; //Power regeneration rate
	short mpow; //Maximum power level
	short shd; //Shield level
	short rshd; //Shield regeneration rate
	short mshd; //Maximum shield level
	long ful; //Fuel level
	long mful; //Maximum fuel level
	short shon; //Shields on?
	short clon; //Cloak on?
	short cpow; //Cloak power consumption
	short frnd; //Friend target
	short enem; //Enemy target
	short plnt; //Planetary target
	struct slot
	{
		pol pos; //Position (ang=-1 indicates not external)
		short face; //Facing direction
		short indx; //Equipment item index
		short rdy; //Readiness
		short cap; //Capacity
	}	
	slots[32]; //Equipment
	short mlck; //Mass locked?
	long cash; //Cash onboard

	long hrx,hry; //Hi-res X, subdivides the co-ords into -500 - 500. NOT to be used by any function except 'physics', allows smooth movement at low speed
	vct vect; //Caches velocity vector for use by other functions
	long hra; //Hi-res angle
	long hrsp; //Hi-res speed, subdivided -500-500. Only to be used by the 'accel' function
	
	void draw(); //Draw ship
	void physics(); //Execute physics on the ship, such as motion
	void turn(short dir); //Turns the ship by specified amount`
	void accel(long acc,bool wrp); //Accelerates the ship by specified amount, wrp deciding if you are willing to make the impulse<=>warp transition
	void autonav(); //Executes auto-pilot considerations on ship
	void save(FILE* strm); //Saves the ship data to a filestream
	void load(FILE* strm); //Load ship data from a filestream
	void shoot(short wep,bool aim); //Shoot at target with wep: 1 Phasers  2 Torpedo .'aim' determines if enhanced aiming will be active for phasers 
	void social(); //Make targetting decisions
	void maintain(); //Does routine work such as recharging shields and power, reloading weapons
	void detonate(frag* in,bool deb); //Detonates the ship as a consequence of incoming frag. Solid debris?
	bool see(short indx); //Can the ship see the ship given by index? (sensor range stuff)
	void kill(); //Kill ship and do necessary cleanup
	void disown(); //Disown of all friends
	void colldetect(frag* frg); //Checks a given frag for collision and takes appropriate action
	void cloak();
	void uncloak();
	void shieldsup();
	void shieldsdown(); //Self explanatory (includes sanity checks)
	vct getpoint(short p); //Get the full position of an equipment point
	void giveorder(short typ,bool bcst); //This ship issues order to fleet with typ: 1 Hold position  2 Form up  3 Disband  4 Concentrate fire  5 Capture/transfer command. 'bcst' determines if it's a broadcast or local
	void defcheck(); //Rouse defenders to target (enemy) planet 
	void mlckcheck(); //Update mass lock
	void scan(bool diag); //Scan and printout in the console, is it a self diagnostic?
	long buy(short prch,bool doit); //Ship trys to purchase given item off target planet, cost is returned. Purchase only takes please if 'doit' is true
	bool beam(void* trg,short typ); //Attempt transport to given target object, typ 1=Ship 2=Planet, returns if successful
	short freemass(); //Returns free mass on board
	void slotrotate(short typ); //Rotates around the equipment in slots on board. typ: 1 Rotates used slots  2 Rotates free external slots
	void jettison(); //Jettisons first slot of equipment
	void repair(); //Repairs first slot of equipment
};
enum STAT {STAT_CRIPPLED,STAT_HOLDING,STAT_ACTIVE}; //Ship status type


class equip //Equipment item type
{
	public:
	
	short self; //Self index in the equipment cache
	char nam[65]; //Name
	short typ; //Type  1:Energy weapon 2:Missile launcher 3:Transporter 4:Hangar 5:Cloaking device
	short mss; //Mass
	short spr; //Associated sprite index
	short snd; //Associated sound index
	short pow; //Power consumption
	short rdy; //Readiness cycle time
	short cap; //Capacity
	short rng; //Range
	short trck; //Tracking power
	short acov; //Angle coverage
	long cost; //Base cost
	
	void load(); //Load equipment data from the database
};

class planet //Planet object
{
	public:
	
	short self; //Self index
	char nam[65]; //Name
	short spr; //Sprite index
	short rot; //Version of sprite to use
	short all; //Alliance (-1 means non-existent)
	long x,y; //Co-ordinates
	short typ; //Planet type 1:Sun 2:Uninhabited 3:Inhabited
	short sold[8]; //Sold equipment
	
	void draw(); //Draw the planet
	void scan(ship* mine); //Scans the planet onto the console
	void stockup(); //Stocks up the planetary equipment market
	void save(FILE* strm); //Save planet to a filestream
	void load(FILE* strm); //Load planet from a filestream
};

class ali //Alliance type
{
	public:
	
	short self; //Self index
	char nam[65]; //Name
	short grp; //Ally group
	short form; //Formation discipline (during fights)
	short flts; //Fleet size
	short aggr; //Aggression
	short defn; //Defensiveness
	short cour; //Courage
	short nsys; //Number of star systems
	short pinh; //Percentage of inhabited planets
	short trav[16]; //Alliances to commute to, density distribution
	short sdns[16]; //Ship density distribution
	short spw; //Spawned one yet? (used to guarantee a ship in each alliance)
	char cmst[16][65]; //Comm strings
	long wlth; //Wealth, average cash on ship
	long ripo; //Ripoff percentage for purchases (100 is normal)
	short trad; //Trading standards; 0 - Will trade to no-one  1 - Will trade to anyone  2 - Will trade to allies  3 - Only trade with own lot
	short eqps[16]; //Available equipment	
		
	void load(); //Load alliance from datafile
	short getspawn(long salt); //Returns a ship index from the database of that alliance
};

struct event //Game user event
{
	public:

	short ship; //Associated ship index
	short act; //Action
	short opr; //Action operand

	void execute(); //Runs the action
};
enum ACT {ACT_ACC,ACT_TURN,ACT_SHIPTARGET,ACT_PLANTARGET,ACT_SHOOT,ACT_BUY,ACT_ORDER}; //Action type



const short ALICOUNT=8; //Number of alliances
const short SHIPCOUNT=128; //Total number of ships to be implemented in the game
const short SHIPDBCOUNT=16; //Total number of ship types in database
const short EQUIPCOUNT=64; //Maximum number of equipment items in the database
const short FRAGCOUNT=512; //Total number of frags in database
const short PLANETCOUNT=256; //Total number of planets to be implemented
