/*
	graphics.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include "SDL_rotozoom.h"
#include "system.h"
#include "config.h"
#include "defs.h"
#include "public.h"
#include "layout.h"

class sprite //A sprite object
{
	public:	
	
	short self; //Self index in the sprite cache
	SDL_Surface* rots[36]; //Rotations. rot[0] loaded from disk, others calculated
	short hw[36],hh[36]; //Half height and half width for each rotation. Quick calculations and all that...	
	bool imem[36]; //In memory and loaded?
	
	void load(short rot,short mode); //Loads the sprite from the datafiles. Mode: 0 Don't calculate rotations  1 Calculate rotations  2 Random munging for debris
	void draw(short x,short y,short rot,short mode); //Displays sprite centred at x,y with the given rotation frame
};

const short SPRITECOUNT=512; //Total number of sprites allowed
Uint16 cols[16]; //Colour keys, defined at initialisation
		//black,red,green,blue,dark grey,light grey,white 
sprite sprites[SPRITECOUNT]; //Main sprite storage bank
SDL_Surface* screen; //Main drawing surface

void initgraphics()
{
	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
	screen=SDL_SetVideoMode(MAIN.w,MAIN.h,0,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_HWPALETTE);
	sdlnulltest((void*)screen);
	SDL_WM_SetCaption("Star Voyager","Star Voyager");
	cols[0]=SDL_MapRGB((*screen).format,0,0,0);
	cols[1]=SDL_MapRGB((*screen).format,255,0,0);
	cols[2]=SDL_MapRGB((*screen).format,0,255,0);
	cols[3]=SDL_MapRGB((*screen).format,0,0,255);
	cols[4]=SDL_MapRGB((*screen).format,180,180,180);
	cols[5]=SDL_MapRGB((*screen).format,64,64,64);
	cols[6]=SDL_MapRGB((*screen).format,255,255,255);
	for(int i=0;i<SPRITECOUNT;i++)
	{
		sprites[i].self=i;
		for(short j=0;j<36;j++)
			sprites[i].imem[j]=false;
	}
	sprites[4].draw(PANEL.x+PANEL.w/2,PANEL.y+PANEL.h/2,0,0);
	SDL_ShowCursor(0);
	sprites[15].load(0,0);
	sprites[6].load(0,0);
	SDL_SetColorKey(sprites[6].rots[0],SDL_SRCCOLORKEY|SDL_RLEACCEL,cols[6]);
}

void update()
{
	SDL_Flip(screen);
}

void string(char* str,short x,short y)
{
	SDL_Rect srct,drct; //Source rect and destination rect
	short i; //Position in text
	char c; //Character in use
	char l; //Letter to print
	bool dr; //Draw it?
	
	i=0;
	srct.y=1;
	srct.w=5;
	srct.h=5;
	drct.x=x;
	drct.y=y;
	drct.w=5;
	drct.h=5;
	while(str[i]!=0)
	{
		c=str[i];
		if(c>32 && c<127)
		{
			l=c-33;
			srct.x=l*6+1;
			SDL_BlitSurface(sprites[15].rots[0],&srct,screen,&drct);
		}
		drct.x+=6;		
		i++;
	}
}

void drawsprite(short indx,short x,short y,short rot,short mode)
{
	sprites[indx].draw(x,y,rot,mode);
}

void getspritedims(short indx,short rot,short mode,short* hw,short* hh)
{
	if(!sprites[indx].imem[rot])
		sprites[indx].load(rot,mode);	
	*hw=sprites[indx].hw[rot];
	*hh=sprites[indx].hh[rot];
}

void sprite::load(short rot,short mode)
{
	SDL_Surface* tmp; //Temporary holding place while sprites are converted
	char* path; //Path to load sprite from
	double zoom; //Sprite zoom
	short ang; //Actual angle in degrees
	
	imem[rot]=true;
	if(!imem[0])
                load(0,mode);
	zoom=1.0;
	if(rot==0)
	{
		memtest(path=new char[strlen(DATADIR)+strlen(PATHSEP)+32]);
		sprintf(path,"%s%sgfx%s%i.bmp",DATADIR,PATHSEP,PATHSEP,self);
		rots[0]=SDL_LoadBMP(path);
		delete(path);
	}
	ang=360-rot*10;
	if(mode==2)
		zoom=0.5+((double)rnd(100))/100.0;
	if(mode==3)
		zoom=1.0/(36.0/(rot+1));
	if(rot>0)
		rots[rot]=rotozoomSurface(rots[0],ang,zoom,0);
	sdlnulltest((void*)rots[rot]);
	SDL_SetColorKey(rots[rot],SDL_SRCCOLORKEY|SDL_RLEACCEL,cols[0]);
	hw[rot]=(*rots[rot]).w/2;
	hh[rot]=(*rots[rot]).h/2;
	tmp=SDL_ConvertSurface(rots[rot],screen->format,SDL_SWSURFACE);
	SDL_FreeSurface(rots[rot]);
	rots[rot]=tmp;
}

void sprite::draw(short x,short y,short rot,short mode)
{
	SDL_Rect dst; //Destination rect
	
	if(!imem[rot])
		load(rot,mode);
	dst.x=x-hw[rot];
	dst.y=y-hh[rot];
	SDL_BlitSurface(rots[rot],NULL,screen,&dst);
}

void box(long x,long y,long w,long h,short col)
{
	SDL_Rect rect; //Rectangle to draw
	
	rect.x=x;
	rect.y=y;
	rect.w=w;
	rect.h=h;
	SDL_FillRect(screen,&rect,cols[col]);
}

void pix(long x,long y,short col)
{
	SDL_Rect rect; //Rectangle to draw

	rect.x=x;
	rect.y=y;
	rect.w=1;
	rect.h=1;
	SDL_FillRect(screen,&rect,cols[col]);
}

void clip(short x,short y,short w,short h)
{
	SDL_Rect rect; //Rectangle to clip with
	
	rect.x=x;
	rect.y=y;
	rect.w=w;
	rect.h=h;
	SDL_SetClipRect(screen,&rect);
}
