/*
	interface.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include "system.h"
#include "init.h"
#include "defs.h"
#include "public.h"
#include "layout.h"

extern short me;

enum MODE {MODE_STAT,MODE_EQUIP,MODE_SCAN,MODE_STARS,MODE_COMM,MODE_BCAST,MODE_SYS,MODE_CHOOSE,MODE_BUY,MODE_DEAD}; //Console modes
bool flck; //Alternating flicker
Uint8 keys[512]; //Keyboard state
bool rtks[512]; //Realtime keys
short lktm; //Last key-press timer
ship* mine; //User's ship
long povx,povy; //Point of view
pol pntr; //Pointer to target
long stx[64],sty[64],stf[64]; //The stars
long ovel; //Old velocity, used for the purposes of resetting stars after a warp transition
short mode; //Console mode
short ln; //Console line
long rng; //Sensor range
bool anav; //Autonavigation?
bool aim; //Auto aim weapons?
bool cupd; //Console update?
char mstr[128]; //Current message string
short mfrm; //Message sender
short mhs[16]; //Message history
short mhsp; //Message history pointer
short mtm; //Message timer
short recy; //Message history recycle timer
short shkv; //Shake value

void drawstars(); //Draw the stars
void drawsensor(); //Draw the sensor display
void drawbars(); //Draw indicator bars
void drawpointer(); //Draw pointer to target
void drawcons(); //Draw the console if it needs updating
void getkeys(); //Get the keyboard state with repeat rate and suchlike accounted for, stash it in 'keys'
void updateinterface(); //Updates PoV and viewing
void choose(); //Mechanism for choosing ships at the start
void realtimecontrols(); //Controls for movement, navigation and combat
void commcontrols(bool bcst); //Controls for ship orders, bcst determining broadcast or specific
void planetcontrols(); //Controls for planetary communications
void equipcontrols(); //Controls for the equipment manifest
void systemcontrols(); //Controls for quitting and savegames

void initinterface(bool ngam)
{
	ln=0;
	for(short i=0;i<512;i++)
		rtks[i]=false;
	rtks[SDLK_LEFT]=true;
	rtks[SDLK_RIGHT]=true;
	rtks[SDLK_UP]=true;
	rtks[SDLK_DOWN]=true;
	rtks[SDLK_SPACE]=true;
	rtks[SDLK_z]=true;
	rtks[SDLK_LSHIFT]=true;
	rtks[SDLK_RSHIFT]=true;
	rtks[SDLK_MINUS]=true;
	rtks[SDLK_EQUALS]=true;
	lktm=0;
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
	rng=1000;
	anav=false;
	aim=true;
	cupd=true;
	if(ngam)
		mode=MODE_CHOOSE;
	else
		mode=MODE_STAT;
	for(short i=0;i<16;i++)
		mhs[i]=-1;
	mstr[0]='\0';
	mtm=0;
	recy=0;
	for(short i=0;i<64;i++)
	{
		stx[i]=-1;
		stf[i]=1;
	}
	shkv=0;
	updateinterface();
}

void control()
{
	SDL_Event evnt; //Event to be polled
	
	if(!SDL_PollEvent(&evnt))
		evnt.type=0;
	if(evnt.type==SDL_QUIT)
		quit("Successful exit");
	getkeys();
	if(mode==MODE_CHOOSE)
		choose();
	if(mode!=MODE_CHOOSE && mode!=MODE_DEAD)
	{
		realtimecontrols();
		if(mode==MODE_EQUIP)
			equipcontrols();
		if(mode==MODE_COMM && mine->enem!=-1)
			commcontrols(false);
		if((mode==MODE_COMM || mode==MODE_BUY) && mine->plnt!=-1)
			planetcontrols();
		if(mode==MODE_BCAST)
			commcontrols(true);
		if(mode==MODE_SYS)
			systemcontrols();
	}
	if(mode==MODE_DEAD && keys[SDLK_q])
		quit("Successful exit");
	updateinterface();
	if(mine->all==-1 && mode!=MODE_DEAD)
	{
		mode=MODE_DEAD;
		cupd=true;
	}
}

void realtimecontrols()
{
	if(anav && mine->plnt!=-1)
	{
		mine->autonav();
	}
	else
	{		
		if(keys[SDLK_LEFT])
			mine->turn(-32767);
		if(keys[SDLK_RIGHT])
			mine->turn(+32767);
		if(keys[SDLK_UP])
		{
			if(keys[SDLK_LSHIFT] || keys[SDLK_RSHIFT])
				mine->accel(+65535,true);
			else
				mine->accel(+65535,false);		
		}
			if(keys[SDLK_DOWN])
		{
			if(keys[SDLK_LSHIFT] || keys[SDLK_RSHIFT])
				mine->accel(-65535,true);
			else
				mine->accel(-65535,false);
		}
	}
	if(keys[SDLK_SPACE])
	{
		if(aim)
			mine->shoot(1,true);
		else	
			mine->shoot(1,false);
	}
	if(keys[SDLK_z])
		mine->shoot(2,false);
	if(keys[SDLK_t])
	{
		mine->enem=retargetships(mine,1);
		mine->plnt=-1;
		anav=false;
		cupd=true;
		if(mine->enem!=-1)
			playsound(0,1);
	}
	if(keys[SDLK_e])
	{
		mine->enem=retargetships(mine,2);
		mine->plnt=-1;
		anav=false;
		cupd=true;
		if(mine->enem!=-1)
			playsound(3,1);
	}
	if(keys[SDLK_p])
	{
		if(mode==MODE_STARS)
			mine->plnt=retargetplanets(mine,2);
		else
			mine->plnt=retargetplanets(mine,1);
		mine->enem=-1;
		anav=false;
		cupd=true;
		if(mine->plnt!=-1)
			playsound(0,1);
	}
	if(keys[SDLK_s])
		if(mine->shon==1)
			mine->shieldsdown();
		else
			mine->shieldsup();
	if(keys[SDLK_c])
		if(mine->clon>0)
			mine->uncloak();
		else
			mine->cloak();
	if(keys[SDLK_EQUALS])
		rng-=400;
	if(keys[SDLK_MINUS])
		rng+=400;
	for(short i=SDLK_F1;i<=SDLK_F7;i++)
	{
		if(keys[i])
		{
			playsound(1,1);
			mode=i-SDLK_F1;
			cupd=true;
		}
	}
	if(keys[SDLK_n])
	{
		if(mine->plnt!=-1)
			anav=!anav;
		else
			anav=false;
	}
	if(keys[SDLK_a])
	{
		if(aim)
		{
			aim=false;
			quickmessage("Target anticipation off");
		}
		else
		{
			aim=true;
			quickmessage("Target anticipation on");
		}
	}
			
}

void commcontrols(bool bcst)
{
	if(keys[SDLK_1])
		mine->giveorder(1,bcst);
	if(keys[SDLK_2])
		mine->giveorder(2,bcst);
	if(keys[SDLK_3])
		mine->giveorder(3,bcst);
	if(keys[SDLK_4])
	{
		if(bcst)
			mine->giveorder(4,bcst);
		else
			mine->giveorder(5,bcst);
	}
}

void planetcontrols()
{
	if(mode==MODE_BUY)
	{
		for(short i=SDLK_1;i<=SDLK_8;i++)
		{
			if(keys[i])
			{
				mine->buy(i-SDLK_1,true);
				break;
			}
		}
	}
	if(mode==MODE_COMM)
	{
		if(keys[SDLK_1])
		{
			mine->buy(-1,true);
			cupd=true;
		}
		if(keys[SDLK_2])
		{
			mine->buy(-2,true);
			cupd=true;
		}
		if(keys[SDLK_3])
		{
			mine->buy(-3,true);
			cupd=true;
		}
		if(keys[SDLK_4])
		{
			mode=MODE_BUY;
			cupd=true;
		}
	}

}

void equipcontrols()
{
	if(keys[SDLK_1])
	{
		mine->slotrotate(1);
		cupd=true;
	}
	if(keys[SDLK_2])
	{
		mine->slotrotate(2);
		cupd=true;
	}
	if(keys[SDLK_3])
	{
		mine->jettison();
		cupd=true;
	}
	if(keys[SDLK_4])
	{
		mine->repair();
		cupd=true;
	}
}

void systemcontrols()
{
	if(keys[SDLK_1])
		quit("Successful exit");
	if(keys[SDLK_2])
		savegame();
	if(keys[SDLK_3])
		loadgame();
	if(keys[SDLK_4])
		mine->cash=999999;
	if(keys[SDLK_5])
	{
		if(togglesound())
			quickmessage("Sound on");
		else
			quickmessage("Sound off");
	}
}

void updateinterface()
{
	vct vtrg; //Vector to target
	ship* tshp; //Ship target
	planet* tpln; //Planet target
	long rad; //Radius for calculating PoV

	mine=(ship*)getship(me);
	vtrg.xx=0;
	vtrg.yy=0;
	if(mine->plnt!=-1)
	{
		tpln=getplanet(mine->plnt);
		vtrg.xx=tpln->x-mine->x;
		vtrg.yy=tpln->y-mine->y;
	}
	if(mine->enem!=-1)
	{
		tshp=getship(mine->enem);
		vtrg.xx=tshp->x-mine->x;
		vtrg.yy=tshp->y-mine->y;
	}
	pntr=vcttopol(vtrg);
	vtrg.xx=0;
	vtrg.yy=0;
	if(mine->enem!=-1)
	{
		rad=pntr.rad;
		if(rad<100)
			rad=100;
		vtrg.xx=100*(tshp->x-mine->x)/rad;
		vtrg.yy=100*(tshp->y-mine->y)/rad;
	}
	povx=mine->x+vtrg.xx;
	povy=mine->y+vtrg.yy;
	if(shkv>0)
	{
		povx+=rnd(shkv)-shkv/2;
		povy+=rnd(shkv)-shkv/2;
		shkv-=2;
	}
	else
	{
		shkv=0;
	}
	if(rng<1000)
		rng=1000;
	if(rng>mine->srng)
		rng=mine->srng;
}

void drawinterface()
{
	drawsensor();
	drawbars();
	if(cupd)
		drawcons();
	clip(VIEW.x,VIEW.y,VIEW.w,VIEW.h);
	box(VIEW.x,VIEW.y,VIEW.w,VIEW.h,0);
	drawstars();
	drawpointer();
}

void drawstars()
{
	short n; //Loop limit
	long astx,asty; //Actual screen positions
	
	n=64-mine->vel.rad/3000;
	for(short i=0;i<n;i++)
	{
		astx=stx[i]+(VIEW.x+VIEW.w/2-povx)/stf[i];
		asty=sty[i]+(VIEW.y+VIEW.h/2-povy)/stf[i];
		if(astx<VIEW.x || astx>VIEW.x+VIEW.w || asty<VIEW.y || asty>VIEW.y+VIEW.h || (ovel>99 && mine->vel.rad<100))
		{
			astx=VIEW.x+rnd(VIEW.w);
			asty=VIEW.y+rnd(VIEW.h);
			if(mine->vel.rad>99)
				stf[i]=1+rnd(2000);		
			else
				stf[i]=1+rnd(10);
			stx[i]=astx-(VIEW.x+VIEW.w/2-povx)/stf[i];
			sty[i]=asty-(VIEW.y+VIEW.h/2-povy)/stf[i];
		}
		if(mine->vel.rad>99)
			drawsprite(3,astx,asty,((mine->vel.ang+5)/10)%36,1);
		else
		{
			if(stf[i]>5)
				pix(astx,asty,4);
			else
				pix(astx,asty,6);
		}
	}
	ovel=mine->vel.rad;
}

void drawsensor()
{
	short locw; //Local area width
	char cord[64]; //Co-ordinate string
	
	flck=!flck;
	clip(RADAR.x,RADAR.y,RADAR.w,RADAR.h);
	box(RADAR.x,RADAR.y,RADAR.w,RADAR.h,0);
	sprintf(cord,"%ld , %ld",mine->x/100,mine->y/100);
	string(cord,RADAR.x+1,RADAR.y+1);
	for(long x=(mine->x-rng)/2000,l=(mine->x+rng)/2000,ax=0;x<=l;x++)
	{
		ax=(((x*2000)-mine->x)*(RADAR.w/2))/rng+RADAR.x+RADAR.w/2;
		box(ax,RADAR.y+RADAR.h-2,1,3,5);
	}
	for(long y=(mine->y-rng)/2000,l=(mine->y+rng)/2000,ay=0;y<=l;y++)
	{
		ay=(((y*2000)-mine->y)*(RADAR.h/2))/rng+RADAR.y+RADAR.h/2;
		box(RADAR.x+RADAR.w-2,ay,3,1,5);
	}
	locw=(200*(RADAR.w/2))/rng;
	box(RADAR.x+RADAR.w/2-locw,RADAR.y+RADAR.h/2-locw,2*locw,2*locw,5);
	if(flck)
		pix(RADAR.x+RADAR.w/2,RADAR.y+RADAR.h/2,6);		
	radarships(mine->x,mine->y,rng,mfrm);
	radarfrags(mine->x,mine->y,rng);
	radarplanets(mine->x,mine->y,rng,mine->plnt);
	
}

void drawbars()
{
	long h,y; //Height and y position of bar

	clip(BARS.x,BARS.y,BARS.w,BARS.h);
	box(BARS.x,BARS.y,BARS.w,BARS.h,0);
	y=-(mine->shd*BARS.h)/(mine->mshd+1)+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x,y,8,h,1);
	y=-(mine->pow*BARS.h)/(mine->mpow+1)+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+8,y,8,h,2);
	y=-(mine->hul*BARS.h)/(mine->mss+1)+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+16,y,8,h,6);
	y=-(mine->ful*BARS.h)/(mine->mful+1)+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+24,y,8,h,3);
}

void drawcons()
{
	char txt[64]; //Scratchpad
	planet* tpln; //Target planet

	cupd=false;
	clip(CONS.x,CONS.y,CONS.w,CONS.h);
	printcons(NULL);
	if(mode==MODE_STAT)
	{
		printcons("Status");
		printcons("");
		mine->scan(true);
		cupd=true;
	}
	if(mode==MODE_EQUIP)
	{
		printcons("Equipment manifest");
		printcons("");
		manifest(mine);
		printcons("");
		printcons("1) Rotate equipment");
		printcons("2) Rotate free slots");
		printcons("3) Jettison top item");
		printcons("4) Repair top item");
		cupd=true;
	}
	if(mode==MODE_SCAN)
	{
		if(mine->enem==-1 && mine->plnt==-1)
		{
			printcons("No target selected");
			spritecons(5);
		}
		else
		{
			printcons("Scan");
			printcons("");
			if(mine->enem!=-1)
			{
				getship(mine->enem)->scan(false);
				cupd=true;
			}
			if(mine->plnt!=-1)
				getplanet(mine->plnt)->scan(mine);
		}
	}
	if(mode==MODE_STARS)
	{
		printcons("Starchart");
		starchart(mine->x,mine->y,mine->plnt);
		cupd=true;
	}	
	if(mode==MODE_COMM)
	{
		if(mine->plnt!=-1)
		{
			printcons("Planetary communications");
			printcons("");
			sprintf(txt,"1) Refuel %ld",mine->buy(-1,false));
			printcons(txt);
			sprintf(txt,"2) Re-arm %ld",mine->buy(-2,false));
			printcons(txt);
			sprintf(txt,"3) Repair %ld",mine->buy(-3,false));
			printcons(txt);
			printcons("4) Purchase equipment >>>");
		}
		if(mine->enem!=-1)
		{
			printcons("Ship to ship communications");
			printcons("");
			printcons("1) Hold position");
			printcons("2) Form up");
			printcons("3) Disband");	
			printcons("---");
			printcons("4) Transfer of command");
		}
	}
	if(mode==MODE_BCAST)
	{
		printcons("Broadcast communications");
		printcons("");
		printcons("1) Hold position");
		printcons("2) Form up");
		printcons("3) Disband");	
		printcons("4) Concentrate attack pattern");
	}
	if(mode==MODE_SYS)
	{
		printcons("System menu");
		printcons("");
		printcons("1) Quit");
		printcons("2) Save game");
		printcons("3) Load game");
		printcons("4) Cash cheat");
		printcons("5) Toggle sound");
	}
	if(mode==MODE_CHOOSE)
	{
		printcons("Press <TAB> to change alliance");
		printcons("Press return when done");
		printcons("");
		printcons(((ali*)getali(mine->all))->nam);
	}
	if(mode==MODE_BUY && mine->plnt!=-1)
	{
		printcons("Purchase equipment");
		printcons("");
		tpln=getplanet(mine->plnt);
		for(short i=0;i<8 && tpln->sold[i]!=-1;i++)
		{
			sprintf(txt,"%hd) %s %ld",i+1,getequip(tpln->sold[i])->nam,mine->buy(i,false));
			printcons(txt);
		}
	}
	if(mode==MODE_DEAD)
	{
		printcons("Game Over");
		printcons("");
		printcons("Press Q to quit");
	}
}

bool see(long x,long y)
{
	if((x-mine->x)<rng && (x-mine->x)>-rng && (y-mine->y)<rng && (y-mine->y)>-rng)
		return true;
	else
		return false;
}

void drawpointer()
{
	vct vtrg; //Vector to target
	pol ptrg; //Polar to target
	char dsts[16]; //Distance string
		
	ptrg=pntr;
	if(ptrg.rad>VIEW.h/2)
	{
		sprintf(dsts,"%ld",ptrg.rad/100);
		ptrg.rad=VIEW.h/2-30;
		vtrg=poltovct(ptrg,false);
		vtrg.xx+=VIEW.x+VIEW.w/2;
		vtrg.yy+=VIEW.y+VIEW.h/2;
		drawsprite(2,vtrg.xx,vtrg.yy,((ptrg.ang+5)/10)%36,1);	
		string(dsts,vtrg.xx,vtrg.yy+8);
	}
}

void printcons(char* text)
{
	if(text==NULL)
	{
		box(CONS.x,CONS.y,CONS.w,CONS.h,0);
		ln=0;
	}
	else
	{
		string(text,CONS.x+2,CONS.y+ln*6);
		ln++;
	}
	if(ln==26)
		ln=0;
}

void spritecons(short indx)
{
	drawsprite(indx,CONS.x+CONS.w/2,CONS.y+CONS.h/2,0,0);
}

void getreal(long x,long y,long* ax,long* ay)
{
	*ax=x-povx+VIEW.x+VIEW.w/2;
	*ay=y-povy+VIEW.y+VIEW.h/2;
}

void getkeys()
{
	int nkys; //Number of keys
	Uint8* keyb; //Keyboard snapshot
	bool key; //Key press?
	
	SDL_PumpEvents();
	key=false;
	keyb=SDL_GetKeyState(&nkys);
	for(short i=0;i<nkys;i++)
	{
		keys[i]=keyb[i];
		if(!rtks[i] && keys[i])
			key=true;
	}
	if(lktm>0)
	{
		for(short i=0;i<nkys;i++)
		{
			if(!rtks[i])
				keys[i]=false;
		}
	}
	else
	{
		if(key)
			lktm=4;
	}	
	lktm--;
	if(lktm<0)
		lktm=0;
}

void choose()
{
	short all; //Alliance to try
	
	if(keys[SDLK_TAB] || mine->all==-1)
	{
		all=mine->all;
		me=-1;
		for(short i=0;i<ALICOUNT && me==-1;i++)
		{
			all=(all+1)%ALICOUNT;
			me=startme(all);
		}
		cupd=true;
		playsound(1,1);
	}
	if(keys[SDLK_RETURN])
	{
		mode=MODE_STAT;
		cupd=true;
		mine->ply=0;
		mine->enem=-1;
		mine->frnd=-1;
		all=mine->all;
		mine->disown();
		mine->all=all;
		playsound(0,1);
	}
	if(keys[SDLK_a])
		armageddon(mine->plnt);
}

void message(ship* frm,short str)
{
	short hval; //History value
	ali* tali; //Ship alliance
		
	tali=(ali*)getali(frm->all);	
	hval=frm->all*16+str;
	if(mtm>50)
		return;
	for(short i=0;i<16;i++)
	{
		if(mhs[i]==hval)
			return;
	}
	if(!mine->see(frm->self))
		return;
	mtm=100;
	mfrm=frm->self;
	mhsp=(mhsp+1)%16;
	mhs[mhsp]=hval;
	sprintf(mstr,"\"%s\"",tali->cmst[str*2+frm->self%2]);	
	if(mstr[1]=='\"')
		mstr[0]=0;
	if(str==1)
		playsound(2,1);
	else
		playsound(4,1);
}

void quickmessage(char* txt)
{
	mfrm=-1;
	mtm=100;
	sprintf(mstr,"%s",txt);
}

void domessage()
{
	if(mtm>0)
	{	
		string(mstr,VIEW.x+32,VIEW.y+VIEW.h-16);
		mtm--;
	}
	else
		mfrm=-1;
	if(recy>0)
		recy--;
	else
	{
		recy=50;
		mhsp=(mhsp+1)%16;
		mhs[mhsp]=-1;
	}
}

void noise(long x,long y,short indx)
{
	vct ssrc; //Sound source vector
	long dis; //Rough distance from PoV
	
	ssrc.xx=x-povx;
	ssrc.yy=y-povy;
	if(ssrc.xx<0)
		ssrc.xx=-ssrc.xx;
	if(ssrc.yy<0)
		ssrc.yy=-ssrc.yy;
	dis=ssrc.xx+ssrc.yy;
	if(dis<VIEW.w)
	{
		playsound(indx,1);
		return;
	}
	if(dis<VIEW.w*4)
	{
		playsound(indx,3);
		return;
	}
	if(dis<VIEW.w*8)
	{
		playsound(indx,6);
		return;
	}	
}

void shakeme(short pow)
{
	if(pow>2)
		shkv=pow;
	else
		shkv=2;
}
