/*
	planet.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include "system.h"
#include "defs.h"
#include "public.h"
#include "layout.h"

//extern short me;
extern bool flck;

char n1[][16]={"Vul","Et","Fer","Man","Clo","Jen","Reb","Joy","Dan","Sat","Mat","Cor","Wat","An","Oct","Ack"};
char n2[][16]={"tan","can","an","en","in","lat","pil","op","ep","ast","tyn","syn","blat","av","",""};
char n3[][16]={"is","ia","ol","ion","on","ath","ur","e","ise","at","","","","","",""};
planet plns[PLANETCOUNT]; //Planets in game
short slo=100,shi=103,ulo=110,uhi=116,ilo=120,ihi=124; //Boundings for planet sprites

void newplanet(planet* in); //Puts the given planet into existence
void generatename(char* put); //Generates a planet name, inserts into put

void initplanets()
{
	for(short i=0;i<PLANETCOUNT;i++)
		plns[i].all=-1;
}

void newsystem(long x,long y,short all)
{
	planet npln; //New planet scratchpad
	short n; //Number of planets in system
	ali* tali; //System alliance
	char snam[33]; //System name
	
	tali=(ali*)getali(all);
	n=rnd(10)+2;
	npln.all=all;
	npln.x=x;
	npln.y=y;
	for(short i=0;i<n;i++)
	{
		npln.rot=rnd(36);
		if(i==0)
		{
			npln.typ=1;
			npln.spr=slo+rnd(shi-slo+1);
			generatename(npln.nam);
			strcpy(snam,npln.nam);
		}
		else
		{
			if(rnd(100)<tali->pinh || i<3)
			{
				npln.typ=3;
				npln.spr=ilo+rnd(ihi-ilo+1);
				generatename(npln.nam);
			}
			else
			{
				npln.typ=2;
				npln.spr=ulo+rnd(uhi-ulo+1);
				sprintf(npln.nam,"%s %hd",snam,i);
			}
		}			
		newplanet(&npln);
		npln.x=x+rnd(16000)-8000;
		npln.y=y+rnd(16000)-8000;
	}
}

void generatename(char* put)
{
	short s1,s2,s3; //Syllable selectors

	s1=rnd(16);
	s2=rnd(16);
	s3=rnd(16);
	sprintf(put,"%s%s%s",n1[s1],n2[s2],n3[s3]);
}

void newplanet(planet* in)
{
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all==-1)
		{
			plns[i]=*in;
			plns[i].self=i;
			for(short j=0;j<8;j++)
				plns[i].sold[j]=-1;
			if(plns[i].typ==3)
				plns[i].stockup();
			return;
		}
	}
}

void drawplanets()
{
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1)
			plns[i].draw();
	}
}

planet* getplanet(short indx)
{
	return &plns[indx];
}

void radarplanets(long povx,long povy,long rng,short trg)
{
	long ax,ay; //Actual position to draw at

	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1)
		{
			ax=plns[i].x-povx;
			ay=plns[i].y-povy;
			if(ax<rng && ax>-rng && ay<rng && ay>-rng)
			{
				ax=(ax*RADAR.w/2)/rng+RADAR.x+RADAR.w/2;
				ay=(ay*RADAR.h/2)/rng+RADAR.y+RADAR.h/2;
				if(i==trg)
					box(ax-1,ay-1,3,3,3);
				pix(ax,ay,2);
			}
		}
	}
}


void starchart(long povx,long povy,short trg)
{
	long ax,ay; //Actual position to draw at
	ali* tali; //Alliance of targetted planet
	
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1 && plns[i].typ==1)
		{
			ax=plns[i].x/(LIMIT/(CONS.w/2))+CONS.x+CONS.w/2;
			ay=plns[i].y/(LIMIT/(CONS.h/2))+CONS.y+CONS.h/2;
			if(i==trg)
			{
				tali=(ali*)getali(plns[i].all);
				box(ax-1,ay-1,3,3,3);
				string(plns[i].nam,ax+2,ay+2);
				string(tali->nam,ax+2,ay+8);
			}
			pix(ax,ay,2);
		}
	}
	ax=povx/(LIMIT/(CONS.w/2))+CONS.x+CONS.w/2;
	ay=povy/(LIMIT/(CONS.h/2))+CONS.y+CONS.h/2;
	box(ax-1,ay-1,3,3,6);
}

short retargetplanets(ship* mine,short typ)
{
	short t; //Target select

	t=((ship*)mine)->plnt;
	if(t==-1)
		t=0;
	for(short i=0;i<PLANETCOUNT;i++)
	{
		t=(t+1)%PLANETCOUNT;
		if(plns[t].all!=-1)
		{
			if((see(plns[t].x,plns[t].y) && typ==1) || (plns[t].typ==1 && typ==2))
				break;
		}
		if(i==PLANETCOUNT-1)
		{
			t=-1;
			break;
		}		
	}
	return t;
}

void initspawn()
{
	for(short i=0;i<ALICOUNT;i++)
	{
		if(getali(i)->nam[0]!='\0')
		{
			for(short j=0;j<PLANETCOUNT;j++)
			{
				if(plns[j].all==i && plns[j].typ==3)
				{
					shipyard(j);
					break;
				}
			}
		}
	}
}

void shipyard(short indx)
{
	short shp; //Ship currently being added
	short fshp; //Ship currently being added to fleet
	ali* tali; //Alliance
	
	if(plns[indx].typ==3)
	{
		tali=(ali*)getali(plns[indx].all);
		shp=newship(tali->getspawn(indx),plns[indx].x+prnd(100)-50,plns[indx].y+prnd(100)-50,plns[indx].all);
		(getship(shp))->plnt=indx;
		for(short i=0,n=prnd(tali->flts);i<n;i++)
		{
			fshp=newship(tali->getspawn(5*i*indx+5),plns[indx].x+prnd(100)-50,plns[indx].y+prnd(100)-50,plns[indx].all);
			if(fshp!=-1)
				((ship*)getship(fshp))->frnd=shp;
		}
	}
}

short goodplanet(short all)
{
	short tall; //Travel to alliance
	
	tall=getali(all)->trav[prnd(16)];
	for(short i=0,p;i<PLANETCOUNT;i++)
	{
		p=prnd(PLANETCOUNT);
		if(plns[p].all==tall && plns[p].typ>1)
			return p;
	}
	return -1;
}

short mlckcheck(long x,long y)
{
	long dx,dy; //Co-ordinate differences
	
	for(short i=0;i<PLANETCOUNT;i++)
	{
		dx=x-plns[i].x;
		dy=y-plns[i].y;
		if(dx<10000 && dx>-10000 && dy>-10000 && dy<10000)
			return true;
	}
	return false;
}

void saveplanets(FILE* strm)
{
	for(short i=0;i<PLANETCOUNT;i++)
	{
		fprintf(strm,"Object = Planet%hd\n",i);
		plns[i].save(strm);
	}
}

void loadplanets(FILE* strm)
{
	char obsc[33]; //Object name scratchpad
	
	for(short i=0;i<PLANETCOUNT;i++)
	{
		sprintf(obsc,"Planet%hd",i);
		gotoobject(strm,obsc);
		plns[i].load(strm);
	}
}

void planet::draw()
{
	long ax,ay; //Position on screen
	
	getreal(x,y,&ax,&ay);
	if(!(ax>VIEW.x+VIEW.w+150 || ax<VIEW.x-150 || ay>VIEW.y+VIEW.h+150 || ay<VIEW.y-150))
		drawsprite(spr,ax,ay,rot,2);
}

void planet::scan(ship* mine)
{
	char txt[64]; //Scratchpad	
	ali* tali; //This planet's alliance
	
	tali=(ali*)getali(all);
	printcons(nam);
	if(opposes(mine->all,all))
		sprintf(txt,"Alignment:%s(hostile)",tali->nam);
	else
		sprintf(txt,"Alignment:%s",tali->nam);
	printcons(txt);
	if(typ==1)
		printcons("Star");
	if(typ==2)
		printcons("Uninhabited planet");
	if(typ==3)
		printcons("Inhabited planet");
	spritecons(spr);
}

void planet::stockup()
{
	ali* pali; //Planet's alliance
	ship* lshp; //Ship to leech from
	
	pali=getali(all);
	for(short i=0;i<8;i++)
		sold[i]=-1;
	for(short i=0,j=0;i<8 && j<16 && pali->eqps[j]!=-1;i++)
	{
		sold[i]=pali->eqps[j];
		j+=1+rnd(4);
	}
}

void planet::save(FILE* strm)
{
	if(all==-1)
		return;
	fprintf(strm,"Name = %s\n",nam);
	fprintf(strm,"Sprite = %hd\n",spr);
	fprintf(strm,"SpriteRot = %hd\n",rot);
	fprintf(strm,"Team = %hd\n",all);
	fprintf(strm,"XLoc = %ld\n",x);
	fprintf(strm,"YLoc = %ld\n",y);
	fprintf(strm,"Type = %hd\n",typ);
	for(short i=0;i<8;i++)
	{
		fprintf(strm,"Sold%hd = %hd\n",i,sold[i]);
	}
}

void planet::load(FILE* strm)
{
	char atsc[33]; //Attribute scratchpad
	
	extractstring(strm,"Name",nam);
	spr=extractlong(strm,"Sprite");
	rot=extractlong(strm,"SpriteRot");
	all=extractlong(strm,"Team");
	x=extractlong(strm,"XLoc");
	y=extractlong(strm,"YLoc");
	typ=extractlong(strm,"Type");
	for(short i=0;i<8;i++)
	{
		sprintf(atsc,"Sold%hd",i);
		sold[i]=extractlong(strm,atsc);
	}
}
