/*
	ship.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include "system.h"
#include "defs.h"
#include "public.h"
#include "layout.h"
#include "config.h"

extern short me;
extern bool flck;

ship sdb[SHIPDBCOUNT]; //Ship type database
ship ships[SHIPCOUNT]; //Main ship list
short bstr; //Master behaviour strobe
short istr; //Individual behaviour strobe
short j; //Ship behaviour strobe

void defend(short indx); //Rallies defensive forces to a given index planet

void initships()
{
	char* path; //Path to load ship data from
	FILE* strm; //Filestream to use
	
	j=0;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		ships[i].all=-1;
		ships[i].self=i;
		ships[i].ply=-1;
	}
	memtest(path=new char[strlen(DATADIR)+32]);	
	setquick(false);
	for(short i=0;i<SHIPDBCOUNT;i++)
	{
		sdb[i].cls[0]=0;
		sprintf(path,"%s%sships%s%hd.shp",DATADIR,PATHSEP,PATHSEP,i);
		strm=fopen(path,"r");
		if(strm!=NULL)
		{
			sdb[i].load(strm);
			fclose(strm);
		}
	}
}

void drawships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1)
		{
			ships[i].draw();
		}
	}
}

void doships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1 || ships[i].ply>-1)
			ships[i].physics();
		if(ships[i].all!=-1)
			ships[i].maintain();
	}
}

void doai()
{
	bstr=(bstr+1)%1027;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1 && ships[i].ply==-1 && ships[i].stat!=STAT_CRIPPLED)
		{
			istr=(bstr+i*7)%400;
			ships[i].autonav();
			if(ships[i].enem!=-1)
			{
				if(ships[ships[i].enem].mpow>ships[i].mpow/2)
					ships[i].shoot(2,false);
				if(istr<i+150)
					ships[i].shoot(1,true);
			}
		}	
	}
	for(short i=0;i<3;i++)
	{
		j=(j+1)%SHIPCOUNT;
		istr=(bstr+j*7)%400;
		if(ships[j].all!=-1 && ships[j].stat!=STAT_CRIPPLED)
		{
			if(ships[j].ply==-1)
				ships[j].social();
			ships[j].defcheck();
			ships[j].mlck=mlckcheck(ships[j].x,ships[j].y);
		}
	}
}

short newship(short indx,long x,long y,short all)
{
	char err[33]; //Error string
	ali* tali; //Alliance
	
	tali=getali(all);
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all==-1 && ships[i].ply==-1)
		{
			if(sdb[indx].cls[0]==0)
			{
				sprintf(err,"Ship data index %hd not available",indx);
				quit(err);			
			}
			ships[i]=sdb[indx];
			ships[i].self=i;
			ships[i].ply=-1;
			ships[i].typ=indx;
			ships[i].x=x;
			ships[i].y=y;
			ships[i].all=all;
			ships[i].vel.ang=prnd(360);
			ships[i].vel.rad=0;
			ships[i].vect.xx=0;
			ships[i].vect.yy=0;
			ships[i].hrx=0;
			ships[i].hry=0;
			ships[i].hra=0;
			ships[i].hrsp=0;
			ships[i].shd=0;
			ships[i].pow=ships[i].mpow;
			ships[i].ful=ships[i].mful;
			ships[i].frnd=-1;
			ships[i].enem=-1;
			ships[i].plnt=-1;
			ships[i].stat=STAT_ACTIVE;
			ships[i].hul=ships[i].mss;
			ships[i].shon=0;
			ships[i].clon=0;
			ships[i].mlck=0;
			ships[i].cash=((tali->wlth)*(5+prnd(10)))/10;
			for(short j=0;j<32;j++)
				ships[i].slots[j].rdy=0;
			return i;
		}
	}
	return -1;
}

ship* getship(short indx)
{
	return &ships[indx];
}

void radarships(long povx,long povy,long rng,short comm)
{
	long ax,ay; //Actual position to draw at
	
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1 && ships[i].clon!=40)
		{
			ax=ships[i].x-povx;
			ay=ships[i].y-povy;
			if(ax<rng && ax>-rng && ay<rng && ay>-rng)
			{
				ax=(ax*RADAR.w/2)/rng+RADAR.x+RADAR.w/2;
				ay=(ay*RADAR.h/2)/rng+RADAR.y+RADAR.h/2;
				if(i==ships[me].enem)
				{
					box(ax-1,ay-1,3,3,3);
				}
				if(i==comm && flck)
					box(ax-1,ay-1,3,3,6);
				pix(ax,ay,6);
			}
		}
	}
}

short retargetships(ship* mine,short typ)
{
	short t; //Target select

	t=mine->enem;
	if(t==-1)
		t=0;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		t=(t+1)%SHIPCOUNT;
		if(ships[t].all!=-1 && t!=me && mine->see(t) && see(ships[t].x,ships[t].y))
		{
			if(typ==1 || (typ==2 && opposes(mine->all,ships[t].all)))
			{
				break;
			}
		}
		if(i==SHIPCOUNT-1)
		{
			t=-1;
			break;
		}		
	}
	return t;
}

short startme(short all)
{
	ali* tali; //Alliance
	
	tali=(ali*)getali(all);
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all==all && ships[i].typ==tali->sdns[0])
			return i;
	}
	return -1;
}

void defend(short indx)
{
	short all; //Involved alliance
	short defn; //Defensive percentage
	
	all=getplanet(indx)->all;
	defn=getali(all)->defn;
	for(short i=0,j;i<defn;i++)
	{
		j=(i*indx*3)%SHIPCOUNT;
		if(ships[j].all==all && ships[j].enem==-1 && ships[j].frnd==-1 && ships[j].ply==-1 && ships[j].vel.rad==0)	
		{
			ships[j].plnt=indx;
			break;
		}
	}
}

void saveships(FILE* strm)
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		fprintf(strm,"Object = Ship%hd\n",i);
		ships[i].save(strm);
	}
}

void loadships(FILE* strm)
{
	char obsc[33]; //Object name scratchpad

	for(short i=0;i<SHIPCOUNT;i++)
	{
		sprintf(obsc,"Ship%hd",i);
		gotoobject(strm,obsc);
		ships[i].load(strm);
	}
}

void armageddon(short indx)
{
	for(short i=0;i<SHIPCOUNT;i++)
		ships[i].plnt=indx;
	quickmessage("ARMAGEDDON!!!");
}

void ship::draw()
{
	long ax,ay; //Actual position on screen
	short rot; //Rotation frame to use
	short hw,hh; //Sprite dimensions
	short nc; //Number of cloak sprites
	vct pvct; //For illustrating equipment points
	
	getreal(x,y,&ax,&ay);
	if(!(ax>VIEW.x+VIEW.w+150 || ax<VIEW.x-150 || ay>VIEW.y+VIEW.h+150 || ay<VIEW.y-150))
	{
		rot=((vel.ang+5)/10)%36;
		if(clon==40)
		{
			if(self==me)
				drawsprite(7,ax,ay,rot,1);
		}
		else
		{
			drawsprite(spr,ax,ay,rot,1);

			getspritedims(spr,rot,1,&hw,&hh);

			if(self==ships[me].enem)
			{
				drawsprite(0,ax,ay-hh,0,1);
				drawsprite(0,ax,ay+hh,18,1);
				drawsprite(0,ax-hw,ay,27,1);
				drawsprite(0,ax+hw,ay,9,1);
			}
			if(clon!=0)
			{
				getspritedims(spr,rot,1,&hw,&hh);
				nc=clon;
				if(nc<0)
					nc=-nc;
				for(short i=0;i<nc;i++)
					drawsprite(6,ax+rnd(hw*2)-hw,ay+rnd(hh*2)-hh,0,0);
			}
		}
		/*
		for(short i=0;i<32;i++)
		{
			if(slots[i].pos.ang!=-1&&flck)
			{
				pvct=getpoint(i);
				getreal(pvct.xx,pvct.yy,&ax,&ay);
				box(ax-1,ay-1,3,3,1);
			}
		}
		*/
	}
}

void ship::physics()
{
	vct mov; //Resultant motion
	pol avel; //Adjusted velocity

	if(mlck && vel.rad>99)
		vel.rad=mip;
	avel.ang=vel.ang;
	if(vel.rad>mip && vel.rad<100)
		avel.rad=mip;
	else
		avel.rad=vel.rad;
	mov=poltovct(avel,true);
	hrx+=mov.xx;
	hry+=mov.yy;
	vect.xx=hrx/50;
	vect.yy=hry/50;
	if(hrx<-50)
		hrx=50+hrx%50;
	else
		hrx=hrx%50;
	if(hry<-50)
		hry=50+hry%50;
	else
		hry=hry%50;
	x+=vect.xx;
	y+=vect.yy;
}

void ship::turn(short dir)
{
	if(dir>trn)
		dir=trn;
	if(dir<-trn)
		dir=-trn;
	hra+=dir;
	vel.ang+=hra/50;
	hra=(50+hra)%50;
	vel.ang=(vel.ang+360)%360;
}

void ship::accel(long acc,bool wrp)
{
	long osp; //Old speed
	short pcon; //Power consumption
	bool trns; //Transition?
	
	trns=false;
	osp=vel.rad;
	if(vel.rad>=100)
	{
		if(acc>awp)
			acc=awp;
		if(acc<-awp)
			acc=-awp;
	}
	acc*=100;
	if(vel.rad<=mip)
	{
		if(acc>aip)
			acc=aip;
		if(acc<-aip)
			acc=-aip;
	}
	hrsp+=acc;
	vel.rad+=hrsp/50;
	if(hrsp<-50)
		hrsp=50+hrsp%50;
	else
		hrsp=hrsp%50;
	if(wrp)
	{
		if(osp<=mip && vel.rad>mip)
		{
			vel.rad=100;
			trns=true;
		}
		if(osp>=100 && vel.rad<100)
		{
			vel.rad=mip;
			trns=true;
		}
	}
	else
	{
		if(osp<=mip && vel.rad>mip)
			vel.rad=mip;
		if(osp>=100 && vel.rad<100)
			vel.rad=100;
	}
	if(mlck && vel.rad>99)
		vel.rad=mip;
	if(vel.rad>mwp)
		vel.rad=mwp;
	if(vel.rad<0)
		vel.rad=0;
	if(vel.rad>=100)
		pcon=((vel.rad-osp)*mss)/2000;
	else
		pcon=((vel.rad-osp)*mss)/2;
	if(trns)
		pcon=0;
	if(pcon<0)
		pcon=-pcon;
	if(pow>=pcon)
		pow-=pcon;
	else
		vel.rad=osp;
}

void ship::autonav()
{
	pol dpol; //Desired polar to target
	vct vtmp; //Temporary calculation vector
	vct vtrg; //Vector to target
	short dd; //Directional difference
	short tol; //Angular tolerance
	long acc; //Available acceleration
	bool erat; //eratic approach?
	ship* shp; //Target ship
	planet* pln; //Target planet
	
	shp=NULL;
	pln=NULL;
	erat=false;
	if(plnt!=-1)
	{
		pln=(planet*)getplanet(plnt);
	}
	if(enem!=-1)
	{
		shp=&ships[enem];
		erat=true;
		pln=NULL;
	}
	if(frnd!=-1)
	{
		if(enem==ships[frnd].enem || enem==-1)
		{
			shp=&ships[frnd];
			pln=NULL;
				
		}
		erat=false;
	}
	if(vel.rad<100)
	{
		acc=aip;
	}
	else
	{
		acc=awp*100;
		erat=false;
	}
	if((shp==NULL && pln==NULL) || stat==STAT_HOLDING)
	{
		dpol.ang=vel.ang;
		dpol.rad=0;
		dd=0;
	}
	else
	{
		tol=5+(trn/50);
		if(pln==NULL)
		{
			vtrg.xx=shp->x-x;
			vtrg.yy=shp->y-y;
		}
		else
		{
			vtrg.xx=pln->x-x;
			vtrg.yy=pln->y-y;
		}
		if(!erat)
		{
			vtrg.xx+=(self*497)%800-400;
			vtrg.yy+=(self*273)%800-400;
		}
		dpol=vcttopol(vtrg);
		if(dpol.rad>srng)
			erat=false;
		dd=dpol.ang-vel.ang;
		if(erat)
		{
			tol*=(dpol.rad*10)/srng+1;
			if(dd>180+istr%40)
				dd=dd-360;
			if(dd<-180-istr%40)
				dd=dd+360;
		}
		else
		{
			if(dd>180)
				dd=dd-360;
			if(dd<-180)
				dd=dd+360;
		}
		if(dd<tol && dd>-tol)
		{
			dd=0;
			if(pln==NULL)
			{
				dpol.rad-=vel.rad;
				if(!see(shp->self))
					dpol.rad+=srng/2;
				dpol.rad=quickroot(2*acc)*quickroot(dpol.rad-mss/2-100)/20+shp->vel.rad;
			}
			else
			{
				dpol.rad=quickroot(2*acc)*quickroot(dpol.rad)/20;
			}
		}
		else
		{
			dpol.rad=0;
		}
		if(erat)
			dpol.rad+=((istr-200)%(self+32))/5;
	}
	if(dpol.rad>mip && dpol.rad<100)
		dpol.rad=mip;
	turn(dd*50);
	if(pow>mpow/4)
		accel(dpol.rad-vel.rad,true);
}

void ship::save(FILE* strm)
{
	if(all==-1)
		return;
	fprintf(strm,"Type = %hd\n",typ);
	fprintf(strm,"Class = %s\n",cls);
	fprintf(strm,"HullMass = %hd\n",mss);
	fprintf(strm,"ShipSprite = %hd\n",spr);
	fprintf(strm,"ShieldSprite = %hd\n",sspr);
	fprintf(strm,"ShieldSound = %hd\n",ssnd);
	fprintf(strm,"FragSprite = %hd\n",fspr);
	fprintf(strm,"FragSound = %hd\n",fsnd);
	fprintf(strm,"Team = %hd\n",all);
	fprintf(strm,"Status = %hd\n",stat);
	fprintf(strm,"XLoc = %ld\n",x);
	fprintf(strm,"YLoc = %ld\n",y);
	fprintf(strm,"Heading = %hd\n",vel.ang);
	fprintf(strm,"Speed = %ld\n",vel.rad);
	fprintf(strm,"TurnRate = %hd\n",trn);
	fprintf(strm,"SublightLimit = %ld\n",mip);
	fprintf(strm,"SublightAcceleration = %ld\n",aip);
	fprintf(strm,"WarpLimit = %ld\n",mwp);
	fprintf(strm,"WarpAcceleration = %ld\n",awp);
	fprintf(strm,"SensorRange = %ld\n",srng);
	fprintf(strm,"HullStrength = %hd\n",hul);
	fprintf(strm,"HullMass = %hd\n",mss);
	fprintf(strm,"Power = %hd\n",pow);
	fprintf(strm,"PowerRecharge = %hd\n",rpow);
	fprintf(strm,"PowerLimit = %hd\n",mpow);
	fprintf(strm,"Shield = %hd\n",shd);
	fprintf(strm,"ShieldRecharge = %hd\n",rshd);
	fprintf(strm,"ShieldLimit = %hd\n",mshd);
	fprintf(strm,"Fuel = %hd\n",ful);
	fprintf(strm,"FuelLimit = %hd\n",mful);
	fprintf(strm,"ShieldStatus = %hd\n",shon);
	fprintf(strm,"CloakStatus = %hd\n",clon);
	fprintf(strm,"CloakPowerUsage = %hd\n",cpow);
	fprintf(strm,"FriendTarget = %hd\n",frnd);
	fprintf(strm,"EnemyTarget = %hd\n",enem);
	fprintf(strm,"PlanetTarget = %hd\n",plnt);
	fprintf(strm,"MassLock = %hd\n",mlck);
	fprintf(strm,"Cash = %ld\n",cash);
	fprintf(strm,"Player = %hd\n",ply);
	for(short i=0;i<32;i++)
	{
		fprintf(strm,"Slot%hdAngle = %hd\n",i,slots[i].pos.ang);
		fprintf(strm,"Slot%hdRadius = %ld\n",i,slots[i].pos.rad);
		fprintf(strm,"Slot%hdFace = %hd\n",i,slots[i].face);
		fprintf(strm,"Slot%hdItem = %hd\n",i,slots[i].indx);
		fprintf(strm,"Slot%hdReadiness = %hd\n",i,slots[i].rdy);
		fprintf(strm,"Slot%hdCapacity = %hd\n",i,slots[i].cap);
	}
}

void ship::load(FILE* strm)
{
	char atsc[33]; //Attribute scratchpad
	
	typ=extractlong(strm,"Type");
	extractstring(strm,"Class",cls);
	spr=extractlong(strm,"ShipSprite");
	sspr=extractlong(strm,"ShieldSprite");
	ssnd=extractlong(strm,"ShieldSound");
	fspr=extractlong(strm,"FragSprite");
	fsnd=extractlong(strm,"FragSound");
	all=extractlong(strm,"Team");
	stat=extractlong(strm,"Status");
	x=extractlong(strm,"XLoc");
	y=extractlong(strm,"YLoc");
	vel.ang=extractlong(strm,"Heading");
	vel.rad=extractlong(strm,"Speed");
	trn=extractlong(strm,"TurnRate");
	mip=extractlong(strm,"SublightLimit");
	aip=extractlong(strm,"SublightAcceleration");
	mwp=extractlong(strm,"WarpLimit");
	awp=extractlong(strm,"WarpAcceleration");
	srng=extractlong(strm,"SensorRange");
	hul=extractlong(strm,"HullStrength");
	mss=extractlong(strm,"HullMass");
	pow=extractlong(strm,"Power");
	rpow=extractlong(strm,"PowerRecharge");
	mpow=extractlong(strm,"PowerLimit");
	shd=extractlong(strm,"Shield");
	rshd=extractlong(strm,"ShieldRecharge");
	mshd=extractlong(strm,"ShieldLimit");
	ful=extractlong(strm,"Fuel");
	mful=extractlong(strm,"FuelLimit");
	shon=extractlong(strm,"ShieldStatus");
	clon=extractlong(strm,"CloakStatus");
	cpow=extractlong(strm,"CloakPowerUsage");
	frnd=extractlong(strm,"FriendTarget");
	enem=extractlong(strm,"EnemyTarget");
	plnt=extractlong(strm,"PlanetTarget");
	mlck=extractlong(strm,"MassLock");
	cash=extractlong(strm,"Cash");
	ply=extractlong(strm,"Player");
	for(short i=0;i<32;i++)
	{
		sprintf(atsc,"Slot%hdAngle",i);
		slots[i].pos.ang=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdRadius",i);
		slots[i].pos.rad=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdFace",i);
		slots[i].face=extractlong(strm,atsc);	
		if(slots[i].face==-1)
			slots[i].face=slots[i].pos.ang;
		sprintf(atsc,"Slot%hdItem",i);
		slots[i].indx=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdReadiness",i);
		slots[i].rdy=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdCapacity",i);
		slots[i].cap=extractlong(strm,atsc);
	}
	hrx=0;
	hry=0;
	hrsp=0;
	hra=0;
	vect.xx=0;
	vect.yy=0;
}

void ship::shoot(short wep,bool aim)
{
	equip* foc; //Equipment item in focus
	frag out; //Frag scratchpad
	vct vtmp; //Vector scratchpad
	pol ptmp; //Polar scratchpad
	pol ttrg,trg; //Target
	short acov; //Angle coverage
	short dif; //Angle difference
	short rng; //Range
	bool cov; //Covered by angle, range and power?		
	
	if(enem==-1)
		return;
	if(vel.rad>99)
		return;
	if(!see(enem))
		return;
	vtmp.xx=ships[enem].x-x;
	vtmp.yy=ships[enem].y-y;
	ttrg=vcttopol(vtmp);
	trg=ttrg;
	if(aim)
	{
		vtmp.xx+=((ships[enem].vect.xx-vect.xx)*trg.rad)/25;
		vtmp.yy+=((ships[enem].vect.yy-vect.yy)*trg.rad)/25;
		trg=vcttopol(vtmp);	
	}
	for(short i=0;i<32;i++)
	{
		if(slots[i].indx!=-1)
		{
			foc=(equip*)getequip(slots[i].indx);
			cov=false;
			if(wep==1)
			{
				rng=foc->rng*foc->trck;
			}
			if(wep==2)
				rng=(srng*10)/9;
			if(trg.rad<=rng)
			{
				cov=true;
				dif=trg.ang-(vel.ang+slots[i].face);
				if(dif>180)
					dif=dif-360;
				if(dif<-180)
					dif=dif+360;
				if(dif>(foc->acov) || dif<-(foc->acov))
					cov=false;	
				dif=ttrg.ang-(vel.ang+slots[i].face);
				if(dif>180)
					dif=dif-360;
				if(dif<-180)
					dif=dif+360;
				if(dif>(foc->acov) || dif<-(foc->acov))
					cov=false;		
				if(wep==1 && pow<foc->pow)
					cov=false;
				if(!slots[i].cap>0 && foc->typ==2)
					cov=false;
			}
			if(cov && slots[i].rdy==0 && foc->typ==wep)
			{
				uncloak();
				if(wep==1)
					pow-=foc->pow;
				vtmp=getpoint(i);
				ptmp.ang=trg.ang;
				ptmp.rad=foc->trck;
				out.x=vtmp.xx;
				out.y=vtmp.yy;
				out.typ=wep;
				out.spr=foc->spr;
				out.vel=poltovct(ptmp,false);
				out.vel.xx+=vect.xx;
				out.vel.yy+=vect.yy;
				out.rot=((trg.ang+5)/10)%36;
				out.rng=foc->rng;
				out.trck=foc->trck;
				out.trg=enem;
				out.pow=foc->pow;
				out.own=self;
				newfrag(&out);
				noise(out.x,out.y,foc->snd);
				slots[i].rdy=foc->rdy;
				if(wep==2)
				{
					slots[i].cap--;
					break;
				}
			}
		}
	}
}

void ship::social()
{
	vct tvct; //Temporary vector
	pol epol; //Polar to enemy
	pol fpol; //Polar to friend
	ali* tali; //This ship's alliance
	
	tali=(ali*)getali(all);
	if(plnt==-1)
	{
		if(frnd==-1)
			plnt=goodplanet(all);
	}
	else
	{
		if(prnd(100)<3 && vel.rad==0 && enem==-1)
			plnt=-1;
		if(vel.rad>=100)
			if(opposes(all,getplanet(plnt)->all))
				cloak();
	}

	if(enem==-1 || prnd(50)==1)
	{
		if(frnd!=-1 && prnd(100)<tali->form)
		{
			if(ships[frnd].enem!=-1)
				if(opposes(all,ships[ships[frnd].enem].all))
					enem=ships[frnd].enem;
		}
		else
		{
			for(short i=0,j,n=tali->aggr;i<n;i++)
			{
				j=(istr+i)%SHIPCOUNT;
				if(ships[j].stat!=STAT_HOLDING)
				{
					if(opposes(all,ships[j].all))
					{
						if(see(j))
						{
							enem=j;
							if(enem==me)
								message(this,1);
							break;
						}
					}
				}
			}
				
		}
	}
	if(enem==-1)
	{
		if(istr==1)
			shieldsdown();
	}
	else
	{
		shieldsup();
		if(frnd==-1)
		{
			if(ships[enem].mpow/(pow+1)>=tali->cour)
			{
				enem=-1;
				plnt=goodplanet(all);
			}
		}
		else
		{
			if(!ships[frnd].see(enem))
				enem=-1;
		}
	}
	if(frnd==-1)
	{
		stat=STAT_ACTIVE;
	}
	else
	{	
		plnt=ships[frnd].plnt;
		if(ships[frnd].stat==STAT_CRIPPLED)
			frnd=-1;
	}
}

void ship::maintain()
{
	for(short i=0;i<32;i++)
	{
		if(slots[i].rdy>0)
			slots[i].rdy--;
	}
	if(shon==1 && pow>=rshd)
	{
		shd+=rshd;
		pow-=rshd;
		if(shd>mshd)
			shd=mshd;
	}
	else
	{
		shd/=2;
	}
	if(clon>0)
	{
		if(pow>=(cpow*mss)/20)
			pow-=(cpow*mss)/20;
		else
			uncloak();
		clon++;
		if(clon>40)
			clon=40;
	}
	if(clon<0)
		clon++;
	if(pow<mpow&&ful>0)
	{
		ful-=rpow;
		pow+=rpow;
	}
	if(pow>mpow)
	{
		ful+=pow-mpow;
		pow=mpow;
	}
	if(pow<0)
		pow=0;
}

void ship::detonate(frag* in,bool deb)
{
	frag out; //Scratchpad for making debris
	short spd; //Speed of explosion

	spd=2+in->pow/mss;
	out.typ=3;
	for(short i=0;i<3;i++)
	{
		out.pow=prnd(9)-5;
		out.spr=fspr;
		out.rot=prnd(36);
		out.rng=prnd(80)+20;
		out.x=in->x;
		out.y=in->y;
		out.vel.xx=vect.xx;
		out.vel.yy=vect.yy;
		if(i==0 || deb==false)
		{
			out.x=(3*(in->x-x)/5)+x-vect.xx;
			out.y=(3*(in->y-y)/5)+y-vect.yy;
			out.spr=1;
			out.rng=prnd(5)+5;
		}
		out.x+=prnd(8)-4;
		out.y+=prnd(8)-4;
		out.vel.xx+=prnd(spd)-spd/2;
		out.vel.yy+=prnd(spd)-spd/2;
		out.trg=-1;
		newfrag(&out);
	}
}

bool ship::see(short indx)
{
	if(indx==-1)
		return false;
	if(ships[indx].clon==40)
		return false;
	if((ships[indx].x-x)>srng)
		return false;
	if((ships[indx].x-x)<-srng)	
		return false;
	if((ships[indx].y-y)>srng)
		return false;
	if((ships[indx].y-y)<-srng)	
		return false;
	return true;		
}

void ship::colldetect(frag* in)
{
	short rot; //Target rotation
	long x1,y1,x2,y2,xx,yy; //Target bounding box
	short hw,hh; //Sprite dimensions
	short ehit; //Equipment hit
	char txt[64]; //Scratchpad
	
	rot=(vel.ang+5)/10;
	xx=(in->vel.xx-vect.xx)/2;
	yy=(in->vel.yy-vect.yy)/2;
	if(xx<0)
		xx=-xx;
	if(yy<0)
		yy=-yy;
	getspritedims(spr,rot,1,&hw,&hh); 
	x1=x-hw-xx;
	y1=y-hh-yy;
	x2=x+hw+xx;
	y2=y+hh+yy;
	if(in->x>x1 && in->x<x2 && in->y>y1 && in->y<y2)
	{
		if(in->own!=-1)
		{
			if(!opposes(all,ships[in->own].all))
			{
				if(ships[in->own].all!=-1)
				{
					ships[in->own].all=7;
					ships[in->own].disown();
				}
			}
/*			if(enem==-1 && ply==-1)
			{
				enem=in->own;
				debug("enem",enem);
			}*/
		}
		uncloak();
		shd-=in->pow;
		if(self==me)
			shakeme(in->pow/20);
		if(shd>0)
		{
			in->vel.xx+=vect.xx;
			in->vel.yy+=vect.yy;
			in->rng=1;
			in->spr=sspr;
			in->rot=prnd(36);
			in->typ=3;
			noise(x,y,ssnd);
		}
		else
		{
			if(frnd==me)
				message(this,3);
			in->rng=0;
			detonate(in,false);
			shd=0;
			ehit=prnd(32);
			if(slots[ehit].indx!=-1)
			{
				slots[ehit].rdy=-1;
				if(self==me)
				{		
					sprintf(txt,"%s damaged",getequip(slots[ehit].indx)->nam);
					quickmessage(txt);
				}
			}
		}
		hul-=(in->pow*4)/(shd+10);
		if(hul<mss/2 && stat!=STAT_CRIPPLED && ply==-1)
		{
			stat=STAT_CRIPPLED;
			shieldsdown();
			frnd=-1;
			plnt=-1;
			enem=-1;
			if(in->own==me)
				quickmessage("Target disabled");
		}
		if(hul<0)
		{
			if(in->own!=-1)
				if(ships[in->own].frnd==me)
					message(&ships[in->own],4);
			hul=0;
			for(short i=0;i<mss/100+2;i++)
			{	
				in->x=x1+prnd(x2-x1);
				in->y=y1+prnd(y2-y1);
				detonate(in,true);
			}
			if(in->own==me)
				quickmessage("Target destroyed");
			kill();
			noise(x,y,fsnd);	
		}
		in->trg=-1;
	}
}

void ship::kill()
{
	all=-1;
	
	disown();
}

void ship::disown()
{
	short nled; //New fleet leader

	frnd=-1;
	nled=-1;
	for(short i=0;i<SHIPCOUNT;i++)		
		if(ships[i].enem==self)
			ships[i].enem=-1;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1 && ships[i].frnd==self)
		{
			nled=i;
			break;
		}
	}
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1 && ships[i].ply==-1)
		{
			if(i==nled)
				ships[i].frnd=-1;
			else	
				if(ships[i].frnd==self)
					ships[i].frnd=nled;
		}
	}
	purgefrags(self);

}

void ship::cloak()
{
	if(clon==0 && cpow>0)
		clon=1;
}

void ship::uncloak()
{
	if(clon>0)
		clon=-clon;
}

void ship::shieldsup()
{
	if(mshd>0)
		shon=1;
}

void ship::shieldsdown()
{
	shon=0;
}

vct ship::getpoint(short p)
{
	pol pos; //Equipment point position
	vct out; //Output value
	
	pos=slots[p].pos;
	pos.ang=(pos.ang+vel.ang)%360;
	out=poltovct(pos,false);
	out.xx+=x;
	out.yy+=y;
	return out;
}

void ship::giveorder(short typ,bool bcst)
{
	char txt[64]; //Scratchpad

	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(!bcst && enem!=-1)
			i=enem;
		if(ships[i].all!=-1)
		{
			if(typ==5)
			{
				if(ships[i].frnd==self)
				{
					ships[i].shieldsdown();
					if(beam(&ships[i],1))
					{
						for(short j=0;j<SHIPCOUNT;j++)
						{
							if(ships[j].frnd==self || j==self)
								ships[j].frnd=i;
						}
						ships[i].ply=ply;
						if(self==me)
						{
							me=i;
							quickmessage("Transfer completed");
						}
						enem=-1;
						plnt=-1;
						ply=-1;
						ships[i].enem=-1;
						ships[i].frnd=-1;
						ships[i].cash+=cash;
						cash=0;
					}
				}
				if(ships[i].stat==STAT_CRIPPLED)
				{
					if(beam(&ships[i],1))
					{
						ships[i].all=all;
						ships[i].disown();
						ships[i].frnd=self;
						ships[i].enem=-1;
						ships[i].plnt=-1;
						ships[i].stat=STAT_ACTIVE;
						cash+=ships[i].cash;
						if(self==me)
						{
							sprintf(txt,"Ship captured and C %ld seized",ships[i].cash);
							quickmessage(txt);
						}
						ships[i].cash=0;
					}
				}
			}
			if(ships[i].frnd==self)
			{
				if(typ==1)
				{
					ships[i].stat=STAT_HOLDING;	
					ships[i].enem=-1;
				}
				if(typ==2)
				{
					ships[i].stat=STAT_ACTIVE;
					ships[i].enem=-1;
				}
				if(typ==3)
				{
					ships[i].stat=STAT_ACTIVE;
					ships[i].frnd=-1;
					ships[i].enem=-1;
				}
				if(typ==4)
				{
					ships[i].stat=STAT_ACTIVE;
					if(opposes(all,ships[enem].all))
						ships[i].enem=enem;
				}
				if(self==me && typ<5)
					message(&ships[i],0);
			}
		}
		if(!bcst)
			i=SHIPCOUNT;
	}
}

void ship::defcheck()
{
	if(plnt!=-1 && vel.rad>100)
		if(opposes(all,getplanet(plnt)->all))
			defend(plnt);
}

void ship::scan(bool diag)
{
	char txt[64]; //Scratchpad
	ali* tali; //This ship's alliance

	if(ships[me].see(self) || diag)
	{
		tali=(ali*)getali(all);
		if(!diag)
			spritecons(spr);
		sprintf(txt,"%s [%hd]",cls,self);
		printcons(txt);
		if(opposes(ships[me].all,all))
			sprintf(txt,"Alignment:%s(hostile)",tali->nam);
		else
			sprintf(txt,"Alignment:%s",tali->nam);
		printcons(txt);
		sprintf(txt,"Speed:%s",getspeed(vel.rad));
		printcons(txt);
		sprintf(txt,"Shields:%hd/%hd",shd,mshd);
		printcons(txt);
		sprintf(txt,"Energy:%hd/%hd",pow,mpow);
		printcons(txt);
		if(diag)
		{
			sprintf(txt,"Hull:%hd/%hd",hul,mss);
			printcons(txt);
			sprintf(txt,"Fuel:%ld/%ld",ful,mful);
			printcons(txt);
			sprintf(txt,"Credits:%ld",cash);
			printcons(txt);
		}
		//sprintf(txt,"%hd",freemass());
		//printcons(txt);
		//sprintf(txt,"enem:%hd frnd:%hd plnt:%hd",enem,frnd,plnt);
		//printcons(txt);
	}
}

long ship::buy(short prch,bool doit)
{
	ali* tali; //Alliance of planet
	planet* tpln; //Planet to trade with
	long cst; //Eventual cost
	long qnt; //Eventual quantity
	char txt[64]; //Scratchpad
	equip* foc; //Equipment in focus
	
	cst=0;
	if(plnt==-1)
		return 0;
	tpln=getplanet(plnt);
	tali=getali(tpln->all);
	if(doit)
	{
		if(tpln->typ!=3)
		{
			if(self==me)
				quickmessage("No response");
			return 0;
		}
		if(tali->trad==0 || (tali->trad==2 && opposes(all,tali->self)) || (tali->trad==3 && all!=tali->self))
		{
			if(opposes(all,tali->self))
				defend(plnt);
			if(self==me)
				quickmessage("Hail ignored");
			return 0;
		}
		if(!beam(tpln,2))
			return 0;
	}
	if(prch==-1)
	{
		qnt=100000;
		if(mful-ful<qnt)
			qnt=mful-ful;		
		cst=(qnt*(tali->ripo))/100000;
		if(doit)
		{
			if(cst>cash)
			{
				if(self==me)
				{
					sprintf(txt,"Insufficient credit (C %ld required)",cst);
					quickmessage(txt);
				}
				cst=0;
			}
			else
			{
				if(self==me)
				{
					sprintf(txt,"Beamed aboard %ld isogrammes of fuel, C %ld paid",qnt,cst);
					quickmessage(txt);
				}
				ful+=qnt;
				pow=mpow;
			}
		}
	}
	if(prch==-2)
	{
		for(short i=0;i<32;i++)
		{
			if(slots[i].indx!=-1)
			{
				foc=getequip(slots[i].indx);
				if(foc->typ==2)
				{
					if(slots[i].cap<foc->cap)
					{
						qnt=foc->cap-slots[i].cap;
						cst=(foc->cost*qnt*tali->ripo)/(foc->cap*400);
						if(doit)
						{
							
							if(cst>cash)
							{
								if(self==me)
								{
									sprintf(txt,"Insufficient credit (C %ld required)",cst);
									quickmessage(txt);
								}
							}
							else
							{
								
								if(self==me)
								{
									sprintf(txt,"%s reloaded, C %ld paid",foc->nam,cst);
									quickmessage(txt);
								}
								slots[i].cap+=qnt;
							}
						}
						break;
					}
				}
			}
		}
	}
	if(prch==-3)
	{
		cst=((mss-hul)*tali->ripo)/100;
		if(doit)
		{
			
			if(hul>=mss)
			{
				if(self==me)
					quickmessage("Hull already at 100% integrity");
				cst=0;
			}
			else
			{
				
				if(cst>cash)
				{
					if(self==me)
					{
						sprintf(txt,"Insufficient credit (C %ld required)",cst);
						quickmessage(txt);
					}
					cst=0;
				}
				else
				{
					if(self==me)
					{
						sprintf(txt,"Hull inspected and overhauled, C %ld paid",cst);
						quickmessage(txt);
					}
					hul=mss;
				}
			}
		}
	}
	if(prch>=0 && prch<8)
	{
		if(tpln->sold[prch]!=-1)
		{
			foc=getequip(tpln->sold[prch]);
			cst=((foc->cost)*(tali->ripo))/100;
			if(doit)
			{
				if(foc->mss>freemass())
				{
					if(self==me)
						quickmessage("Mass limit exceeded");
				}
				else
				{
					for(short i=0;i<32;i++)
					{
						if(slots[i].indx==-1) 
						{
							if((foc->typ>=3 && slots[i].pos.rad==-1) || (foc->typ<3 && slots[i].pos.rad!=-1))
							{
								if(cst>cash)
								{
									if(self==me)
									{
										sprintf(txt,"Insufficient credit (C %ld required)",cst);
										quickmessage(txt);
									}
									cst=0;		
								}
								else
								{
									if(self==me)
									{
										sprintf(txt,"%s installed, C %ld paid",foc->nam,cst);
										quickmessage(txt);
									}
									slots[i].indx=tpln->sold[prch];
									slots[i].cap=foc->cap;
									slots[i].rdy=0;
								}
								break;
							}	
						}
						if(i==31 && self==me)
							quickmessage("No available infrastructure");
					}
				}
			}
		}
	}
	if(doit)
		cash-=cst;
	return cst;
}
	
bool ship::beam(void* trg,short typ)
{
	equip* trns; //Transporter equipment
	vct tvct;
	pol tpol; //Vectors to target
	bool lock; //Lock possible?
	
	lock=true;
	if(typ!=1 && typ!=2)
		return false;
	if(typ==1)
	{
		tvct.xx=((ship*)trg)->x-x;
		tvct.yy=((ship*)trg)->y-y;
	}		
	if(typ==2)
	{
		tvct.xx=((planet*)trg)->x-x;
		tvct.yy=((planet*)trg)->y-y;
	}
	tpol=vcttopol(tvct);
	if(shd!=0 || clon!=0)
		lock=false;
	if(typ==1)
		if(((ship*)trg)->shd!=0 || ((ship*)trg)->clon!=0)
			lock=false;
	if(lock)
	{
		for(short i=0;i<32;i++)
		{
			if(slots[i].rdy==0)
			{
				trns=getequip(slots[i].indx);
				if(trns->typ==3 && tpol.rad<=trns->rng && trns->pow<=pow)
				{
					slots[i].rdy=trns->rdy;
					pow-=trns->pow;
					playsound(trns->snd,1);
					return true;
				}
			}
		}
	}
	if(self==me)
		quickmessage("Unable to establish transporter lock; check range and shields");
	return false;
}

short ship::freemass()
{
	short out; //Outputted free space
	
	out=mss;
	for(short i=0;i<32;i++)
	{
		if(slots[i].indx!=-1)
			out-=getequip(slots[i].indx)->mss;
	}
	return out;
}

void ship::slotrotate(short typ)
{
	short fuse; //First slot in use
	slot swp,swpp; //Slot swapping temporary storage
	
	swpp.indx=-1;
	swpp.pos.rad=-1;
	fuse=-1;
	for(short i=0;i<32;i++)
	{
		if((typ==1 && slots[i].indx!=-1) || (typ==2 && slots[i].indx==-1 && slots[i].pos.rad!=-1))
		{
			if(fuse==-1)
				fuse=i;
			swp=slots[i];
			slots[i]=swpp;
			swpp=swp;
		}
	}
	if(fuse!=-1)
		slots[fuse]=swpp;
}

void ship::jettison()
{
	for(short i=0;i<32;i++)
	{
		if(slots[i].indx!=-1)
		{
			slots[i].indx=-1;
			return;
		}
	}
}

void ship::repair()
{
	char txt[65]; //Scratchpad
	
	if(shd==0 && vel.rad==0 && clon==0)
	{
		for(short i=0;i<32;i++)
		{
			if(slots[i].indx!=-1)
			{
				slots[i].rdy=0;
				pow=0;
				if(self==me)
				{
					sprintf(txt,"%s repaired",getequip(slots[i].indx)->nam);
					quickmessage(txt);
				}
				return;
			}

		}
	}
	else
	{
		if(self==me)
			quickmessage("Shields, cloak and engines must be powered down first");
	}
}
