/*
	sound.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include "system.h"
#include "config.h"
#include "defs.h"
#include "public.h"

class sound //A sound in the database
{
	public:

	short self; //Self index in the sound database
	Uint8* buff; //Audio buffer
	Uint32 blen; //Buffer length
	bool imem; //In memory and loaded?

	void load(); //Loads sound in from appropriate file
	void play(short div); //Schedules sound to play with given div factor A.S.A.P.
};
struct channel //Sound channel
{
	short indx; //Index of sound currently playing
	long pntr; //Playing pointer for each sound in queue
	short div; //Div factor for each sound
};

SDL_AudioSpec want; //Desired audio specification
SDL_AudioSpec spec; //Obtained audio specification

const short SOUNDCOUNT=512;
sound snds[SOUNDCOUNT]; //Sound database
channel chns[16]; //Sound channels
bool son; //Sound on?

void procaudio(void* dumb,Uint8* buff,int blen); //Audio callback function 

void initsound()
{
	want.freq=22050;
	want.format=AUDIO_S16SYS;
	want.channels=1;
	want.samples=512;
	want.callback=procaudio;
	for(short i=0;i<SOUNDCOUNT;i++)
	{
		snds[i].self=i;
		snds[i].imem=false;
	}	
	for(short i=0;i<16;i++)
		chns[i].indx=-1;
	togglesound();
}

bool togglesound()
{
	if(son)
	{
		SDL_CloseAudio();
		son=false;
	}
	else
	{
		if(SDL_OpenAudio(&want,&spec)==-1)
		{
			son=false;
		}
			else
		{
			son=true;
			SDL_PauseAudio(0);
		}
	}
	return son;
}

void playsound(short indx,short div)
{
	if(indx!=-1 && son)
		snds[indx].play(div);
}

void procaudio(void* dumb,Uint8* buff,int blen)
{
	long plen; //Length to blit to stream

	for(short i=0;i<16;i++)
	{
		if(chns[i].indx!=-1)
		{
			plen=snds[chns[i].indx].blen-chns[i].pntr;
			if(plen>0)
			{
				if(plen>blen)
					plen=blen;
				SDL_MixAudio(buff,&snds[chns[i].indx].buff[chns[i].pntr],plen,SDL_MIX_MAXVOLUME/chns[i].div);
				chns[i].pntr+=plen;
			}
			else
			{
				chns[i].indx=-1;
			}
		}
	}
}

void sound::load()
{
	SDL_AudioSpec wspc; //Wav specification
	SDL_AudioCVT cvt; //Wav to native format conversion structure
	char* path; //Path to load sprite from
	
	memtest(path=new char[strlen(DATADIR)+strlen(PATHSEP)+32]);
	sprintf(path,"%s%ssnd%s%i.wav",DATADIR,PATHSEP,PATHSEP,self);
	sdlnulltest(SDL_LoadWAV(path,&wspc,&buff,&blen));
	delete(path);
	if(SDL_BuildAudioCVT(&cvt,wspc.format,wspc.channels,wspc.freq,spec.format,spec.channels,spec.freq)==-1)
		sdlnulltest(NULL);
	cvt.buf=new Uint8[blen*cvt.len_mult];
	cvt.len=blen;
	memcpy(cvt.buf,buff,cvt.len);
	SDL_ConvertAudio(&cvt);
	SDL_FreeWAV(buff);
	buff=cvt.buf;
	blen=cvt.len_cvt;
	imem=true;
}

void sound::play(short div)
{
	if(!imem)
		load();
	for(short i=0;i<16;i++)
		if(chns[i].indx==self && chns[i].pntr<64)
			return;
	for(short i=0;i<16;i++)
	{
		if(chns[i].indx==-1)
		{
			chns[i].indx=self;
			chns[i].pntr=0;
			chns[i].div=div;
			break;
		}	
	}
}
