/*
	sv.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <string.h>
#include "system.h"
#include "init.h"
#include "defs.h"
#include "public.h"
#include "layout.h"

short me; //My ship index

int main(int argc,char* argv[])
{
	const Uint32 dly=41; //Delay in milliseconds to achieve 24fps
	Uint32 ntm,otm,dif; //Time values, for fps restriction in the loop
	Uint32 stm;
	long frms; //Frame number
	char txt[64]; //FPS text buffer

	initcalc();
	initsound();
	initequip();
	initalis();
	initgraphics();	
	
	initships();
	initfrags();
	initplanets();
	initinterface(true);
	
	inituniverse();
	initspawn();
	me=startme(0);
	frms=0;
	while(true)
	{
		otm=SDL_GetTicks();
		if(frms%100==0)
		{
			stm=SDL_GetTicks();
			frms=0;
		}	
		frms++;
		doships();
		dofrags();
		control();
		doai();
		shipyard(prnd(PLANETCOUNT));
		drawinterface();
		drawplanets();
		drawships();
		drawfrags();
		domessage();
		sprintf(txt,"FPS %f",(float)frms/(((float)otm-(float)stm)/1000));
		string(txt,0,0);
		update();
		ntm=SDL_GetTicks();
		dif=(ntm-otm);
		if(dif<dly)
		{
			dif=dly-dif;
			SDL_Delay(dif);
		}

	}
}

void savegame()
{
	FILE* savf; //Savefile

	savf=fopen("savegame","w+");
	if(savf==NULL)
	{
		quickmessage("Cannot open file");
	}
	else	
	{
		fprintf(savf,"Object = SV021Savegame\n");
		fprintf(savf,"Me = %hd\n",me);
		saveships(savf);
		saveplanets(savf);
		savefrags(savf);
		quickmessage("Game successfully saved");
		fclose(savf);
	}
}

void loadgame()
{
	FILE* savf; //Savefile

	savf=fopen("savegame","r");
	if(savf==NULL)
	{
		quickmessage("Cannot open file");
	}
	else	
	{
		clip(CONS.x,CONS.y,CONS.w,CONS.h);
		printcons(NULL);
		printcons("Loading savegame");
		printcons("");
		update();
		initequip();
		initalis();
		initships();
		initfrags();
		initplanets();
		setquick(true);
		gotoobject(savf,"SV021Savegame");
		me=extractlong(savf,"Me");
		printcons("Loading ships...");
		update();
		loadships(savf);
		printcons("Loading planets...");
		update();
		loadplanets(savf);
		printcons("Loading frags...");
		update();
		loadfrags(savf);
		initinterface(false);
		quickmessage("Game successfully loaded");
		fclose(savf);
	}
}

void debug(const char* str,long num)
{
	printf("%s  %ld\n",str,num);
	fflush(NULL);
}
                
void memtest(void* ptr)
{
	if(ptr==NULL)
		quit("Error allocating dynamic memory");
}

void sdlnulltest(void* test)
{
	char* err; //Error string
	
	if(test==NULL)
	{
		memtest(err=new char[strlen(SDL_GetError())+32]);
		sprintf(err,"SDL Error: %s",SDL_GetError());
		quit(err);
	}
}
                                                                                
void quit(char* msg)
{
	SDL_Quit();
	if(msg!=NULL)
		printf("%s\n",msg);
	exit(0);
}
